# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: enterprise.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='enterprise.proto',
  package='Enterprise',
  syntax='proto3',
  serialized_options=b'\n\030com.keepersecurity.protoB\nEnterprise',
  serialized_pb=b'\n\x10\x65nterprise.proto\x12\nEnterprise\"^\n\x18\x45nterpriseKeyPairRequest\x12\x1b\n\x13\x65nterprisePublicKey\x18\x01 \x01(\x0c\x12%\n\x1d\x65ncryptedEnterprisePrivateKey\x18\x02 \x01(\x0c\"\'\n\x14GetTeamMemberRequest\x12\x0f\n\x07teamUid\x18\x01 \x01(\x0c\"U\n\x0e\x45nterpriseUser\x12\x18\n\x10\x65nterpriseUserId\x18\x01 \x01(\x03\x12\r\n\x05\x65mail\x18\x02 \x01(\t\x12\x1a\n\x12\x65nterpriseUsername\x18\x03 \x01(\t\"K\n\x15GetTeamMemberResponse\x12\x32\n\x0e\x65nterpriseUser\x18\x01 \x03(\x0b\x32\x1a.Enterprise.EnterpriseUser\"-\n\x11\x45nterpriseUserIds\x12\x18\n\x10\x65nterpriseUserId\x18\x01 \x03(\x03\"B\n\x19\x45nterprisePersonalAccount\x12\r\n\x05\x65mail\x18\x01 \x01(\t\x12\x16\n\x0eOBSOLETE_FIELD\x18\x02 \x01(\x0c\"D\n\x17\x45ncryptedTeamKeyRequest\x12\x0f\n\x07teamUid\x18\x01 \x01(\x0c\x12\x18\n\x10\x65ncryptedTeamKey\x18\x02 \x01(\x0c\"+\n\x0fReEncryptedData\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\t\"\xb4\x01\n\x1bNodeToManagedCompanyRequest\x12\x11\n\tcompanyId\x18\x01 \x01(\x05\x12*\n\x05nodes\x18\x02 \x03(\x0b\x32\x1b.Enterprise.ReEncryptedData\x12*\n\x05roles\x18\x03 \x03(\x0b\x32\x1b.Enterprise.ReEncryptedData\x12*\n\x05users\x18\x04 \x03(\x0b\x32\x1b.Enterprise.ReEncryptedData\",\n\x08RoleTeam\x12\x0f\n\x07role_id\x18\x01 \x01(\x03\x12\x0f\n\x07teamUid\x18\x02 \x01(\x0c\"4\n\tRoleTeams\x12\'\n\trole_team\x18\x01 \x03(\x0b\x32\x14.Enterprise.RoleTeamB&\n\x18\x63om.keepersecurity.protoB\nEnterpriseb\x06proto3'
)




_ENTERPRISEKEYPAIRREQUEST = _descriptor.Descriptor(
  name='EnterpriseKeyPairRequest',
  full_name='Enterprise.EnterpriseKeyPairRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='enterprisePublicKey', full_name='Enterprise.EnterpriseKeyPairRequest.enterprisePublicKey', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedEnterprisePrivateKey', full_name='Enterprise.EnterpriseKeyPairRequest.encryptedEnterprisePrivateKey', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=32,
  serialized_end=126,
)


_GETTEAMMEMBERREQUEST = _descriptor.Descriptor(
  name='GetTeamMemberRequest',
  full_name='Enterprise.GetTeamMemberRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='teamUid', full_name='Enterprise.GetTeamMemberRequest.teamUid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=128,
  serialized_end=167,
)


_ENTERPRISEUSER = _descriptor.Descriptor(
  name='EnterpriseUser',
  full_name='Enterprise.EnterpriseUser',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='enterpriseUserId', full_name='Enterprise.EnterpriseUser.enterpriseUserId', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='email', full_name='Enterprise.EnterpriseUser.email', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='enterpriseUsername', full_name='Enterprise.EnterpriseUser.enterpriseUsername', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=169,
  serialized_end=254,
)


_GETTEAMMEMBERRESPONSE = _descriptor.Descriptor(
  name='GetTeamMemberResponse',
  full_name='Enterprise.GetTeamMemberResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='enterpriseUser', full_name='Enterprise.GetTeamMemberResponse.enterpriseUser', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=256,
  serialized_end=331,
)


_ENTERPRISEUSERIDS = _descriptor.Descriptor(
  name='EnterpriseUserIds',
  full_name='Enterprise.EnterpriseUserIds',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='enterpriseUserId', full_name='Enterprise.EnterpriseUserIds.enterpriseUserId', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=333,
  serialized_end=378,
)


_ENTERPRISEPERSONALACCOUNT = _descriptor.Descriptor(
  name='EnterprisePersonalAccount',
  full_name='Enterprise.EnterprisePersonalAccount',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='email', full_name='Enterprise.EnterprisePersonalAccount.email', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='OBSOLETE_FIELD', full_name='Enterprise.EnterprisePersonalAccount.OBSOLETE_FIELD', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=380,
  serialized_end=446,
)


_ENCRYPTEDTEAMKEYREQUEST = _descriptor.Descriptor(
  name='EncryptedTeamKeyRequest',
  full_name='Enterprise.EncryptedTeamKeyRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='teamUid', full_name='Enterprise.EncryptedTeamKeyRequest.teamUid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedTeamKey', full_name='Enterprise.EncryptedTeamKeyRequest.encryptedTeamKey', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=448,
  serialized_end=516,
)


_REENCRYPTEDDATA = _descriptor.Descriptor(
  name='ReEncryptedData',
  full_name='Enterprise.ReEncryptedData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='Enterprise.ReEncryptedData.id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='data', full_name='Enterprise.ReEncryptedData.data', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=518,
  serialized_end=561,
)


_NODETOMANAGEDCOMPANYREQUEST = _descriptor.Descriptor(
  name='NodeToManagedCompanyRequest',
  full_name='Enterprise.NodeToManagedCompanyRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='companyId', full_name='Enterprise.NodeToManagedCompanyRequest.companyId', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nodes', full_name='Enterprise.NodeToManagedCompanyRequest.nodes', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='roles', full_name='Enterprise.NodeToManagedCompanyRequest.roles', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='users', full_name='Enterprise.NodeToManagedCompanyRequest.users', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=564,
  serialized_end=744,
)


_ROLETEAM = _descriptor.Descriptor(
  name='RoleTeam',
  full_name='Enterprise.RoleTeam',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='role_id', full_name='Enterprise.RoleTeam.role_id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='teamUid', full_name='Enterprise.RoleTeam.teamUid', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=746,
  serialized_end=790,
)


_ROLETEAMS = _descriptor.Descriptor(
  name='RoleTeams',
  full_name='Enterprise.RoleTeams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='role_team', full_name='Enterprise.RoleTeams.role_team', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=792,
  serialized_end=844,
)

_GETTEAMMEMBERRESPONSE.fields_by_name['enterpriseUser'].message_type = _ENTERPRISEUSER
_NODETOMANAGEDCOMPANYREQUEST.fields_by_name['nodes'].message_type = _REENCRYPTEDDATA
_NODETOMANAGEDCOMPANYREQUEST.fields_by_name['roles'].message_type = _REENCRYPTEDDATA
_NODETOMANAGEDCOMPANYREQUEST.fields_by_name['users'].message_type = _REENCRYPTEDDATA
_ROLETEAMS.fields_by_name['role_team'].message_type = _ROLETEAM
DESCRIPTOR.message_types_by_name['EnterpriseKeyPairRequest'] = _ENTERPRISEKEYPAIRREQUEST
DESCRIPTOR.message_types_by_name['GetTeamMemberRequest'] = _GETTEAMMEMBERREQUEST
DESCRIPTOR.message_types_by_name['EnterpriseUser'] = _ENTERPRISEUSER
DESCRIPTOR.message_types_by_name['GetTeamMemberResponse'] = _GETTEAMMEMBERRESPONSE
DESCRIPTOR.message_types_by_name['EnterpriseUserIds'] = _ENTERPRISEUSERIDS
DESCRIPTOR.message_types_by_name['EnterprisePersonalAccount'] = _ENTERPRISEPERSONALACCOUNT
DESCRIPTOR.message_types_by_name['EncryptedTeamKeyRequest'] = _ENCRYPTEDTEAMKEYREQUEST
DESCRIPTOR.message_types_by_name['ReEncryptedData'] = _REENCRYPTEDDATA
DESCRIPTOR.message_types_by_name['NodeToManagedCompanyRequest'] = _NODETOMANAGEDCOMPANYREQUEST
DESCRIPTOR.message_types_by_name['RoleTeam'] = _ROLETEAM
DESCRIPTOR.message_types_by_name['RoleTeams'] = _ROLETEAMS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EnterpriseKeyPairRequest = _reflection.GeneratedProtocolMessageType('EnterpriseKeyPairRequest', (_message.Message,), {
  'DESCRIPTOR' : _ENTERPRISEKEYPAIRREQUEST,
  '__module__' : 'enterprise_pb2'
  # @@protoc_insertion_point(class_scope:Enterprise.EnterpriseKeyPairRequest)
  })
_sym_db.RegisterMessage(EnterpriseKeyPairRequest)

GetTeamMemberRequest = _reflection.GeneratedProtocolMessageType('GetTeamMemberRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETTEAMMEMBERREQUEST,
  '__module__' : 'enterprise_pb2'
  # @@protoc_insertion_point(class_scope:Enterprise.GetTeamMemberRequest)
  })
_sym_db.RegisterMessage(GetTeamMemberRequest)

EnterpriseUser = _reflection.GeneratedProtocolMessageType('EnterpriseUser', (_message.Message,), {
  'DESCRIPTOR' : _ENTERPRISEUSER,
  '__module__' : 'enterprise_pb2'
  # @@protoc_insertion_point(class_scope:Enterprise.EnterpriseUser)
  })
_sym_db.RegisterMessage(EnterpriseUser)

GetTeamMemberResponse = _reflection.GeneratedProtocolMessageType('GetTeamMemberResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETTEAMMEMBERRESPONSE,
  '__module__' : 'enterprise_pb2'
  # @@protoc_insertion_point(class_scope:Enterprise.GetTeamMemberResponse)
  })
_sym_db.RegisterMessage(GetTeamMemberResponse)

EnterpriseUserIds = _reflection.GeneratedProtocolMessageType('EnterpriseUserIds', (_message.Message,), {
  'DESCRIPTOR' : _ENTERPRISEUSERIDS,
  '__module__' : 'enterprise_pb2'
  # @@protoc_insertion_point(class_scope:Enterprise.EnterpriseUserIds)
  })
_sym_db.RegisterMessage(EnterpriseUserIds)

EnterprisePersonalAccount = _reflection.GeneratedProtocolMessageType('EnterprisePersonalAccount', (_message.Message,), {
  'DESCRIPTOR' : _ENTERPRISEPERSONALACCOUNT,
  '__module__' : 'enterprise_pb2'
  # @@protoc_insertion_point(class_scope:Enterprise.EnterprisePersonalAccount)
  })
_sym_db.RegisterMessage(EnterprisePersonalAccount)

EncryptedTeamKeyRequest = _reflection.GeneratedProtocolMessageType('EncryptedTeamKeyRequest', (_message.Message,), {
  'DESCRIPTOR' : _ENCRYPTEDTEAMKEYREQUEST,
  '__module__' : 'enterprise_pb2'
  # @@protoc_insertion_point(class_scope:Enterprise.EncryptedTeamKeyRequest)
  })
_sym_db.RegisterMessage(EncryptedTeamKeyRequest)

ReEncryptedData = _reflection.GeneratedProtocolMessageType('ReEncryptedData', (_message.Message,), {
  'DESCRIPTOR' : _REENCRYPTEDDATA,
  '__module__' : 'enterprise_pb2'
  # @@protoc_insertion_point(class_scope:Enterprise.ReEncryptedData)
  })
_sym_db.RegisterMessage(ReEncryptedData)

NodeToManagedCompanyRequest = _reflection.GeneratedProtocolMessageType('NodeToManagedCompanyRequest', (_message.Message,), {
  'DESCRIPTOR' : _NODETOMANAGEDCOMPANYREQUEST,
  '__module__' : 'enterprise_pb2'
  # @@protoc_insertion_point(class_scope:Enterprise.NodeToManagedCompanyRequest)
  })
_sym_db.RegisterMessage(NodeToManagedCompanyRequest)

RoleTeam = _reflection.GeneratedProtocolMessageType('RoleTeam', (_message.Message,), {
  'DESCRIPTOR' : _ROLETEAM,
  '__module__' : 'enterprise_pb2'
  # @@protoc_insertion_point(class_scope:Enterprise.RoleTeam)
  })
_sym_db.RegisterMessage(RoleTeam)

RoleTeams = _reflection.GeneratedProtocolMessageType('RoleTeams', (_message.Message,), {
  'DESCRIPTOR' : _ROLETEAMS,
  '__module__' : 'enterprise_pb2'
  # @@protoc_insertion_point(class_scope:Enterprise.RoleTeams)
  })
_sym_db.RegisterMessage(RoleTeams)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
