# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: folder.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='folder.proto',
  package='Folder',
  syntax='proto3',
  serialized_options=b'\n\030com.keepersecurity.protoB\006Folder',
  serialized_pb=b'\n\x0c\x66older.proto\x12\x06\x46older\"\\\n\x10\x45ncryptedDataKey\x12\x14\n\x0c\x65ncryptedKey\x18\x01 \x01(\x0c\x12\x32\n\x10\x65ncryptedKeyType\x18\x02 \x01(\x0e\x32\x18.Folder.EncryptedKeyType\"\x82\x01\n\x16SharedFolderRecordData\x12\x11\n\tfolderUid\x18\x01 \x01(\x0c\x12\x11\n\trecordUid\x18\x02 \x01(\x0c\x12\x0e\n\x06userId\x18\x03 \x01(\x05\x12\x32\n\x10\x65ncryptedDataKey\x18\x04 \x03(\x0b\x32\x18.Folder.EncryptedDataKey\"\\\n\x1aSharedFolderRecordDataList\x12>\n\x16sharedFolderRecordData\x18\x01 \x03(\x0b\x32\x1e.Folder.SharedFolderRecordData\"_\n\x15SharedFolderRecordFix\x12\x11\n\tfolderUid\x18\x01 \x01(\x0c\x12\x11\n\trecordUid\x18\x02 \x01(\x0c\x12 \n\x18\x65ncryptedRecordFolderKey\x18\x03 \x01(\x0c\"Y\n\x19SharedFolderRecordFixList\x12<\n\x15sharedFolderRecordFix\x18\x01 \x03(\x0b\x32\x1d.Folder.SharedFolderRecordFix\"\xa2\x02\n\rRecordRequest\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12&\n\nrecordType\x18\x02 \x01(\x0e\x32\x12.Folder.RecordType\x12\x12\n\nrecordData\x18\x03 \x01(\x0c\x12\x1a\n\x12\x65ncryptedRecordKey\x18\x04 \x01(\x0c\x12&\n\nfolderType\x18\x05 \x01(\x0e\x32\x12.Folder.FolderType\x12\x12\n\nhowLongAgo\x18\x06 \x01(\x03\x12\x11\n\tfolderUid\x18\x07 \x01(\x0c\x12 \n\x18\x65ncryptedRecordFolderKey\x18\x08 \x01(\x0c\x12\r\n\x05\x65xtra\x18\t \x01(\x0c\x12\x15\n\rnonSharedData\x18\n \x01(\x0c\x12\x0f\n\x07\x66ileIds\x18\x0b \x03(\x03\"E\n\x0eRecordResponse\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12\x10\n\x08revision\x18\x02 \x01(\x03\x12\x0e\n\x06status\x18\x03 \x01(\t\"\x80\x01\n\x12SharedFolderFields\x12\x1b\n\x13\x65ncryptedFolderName\x18\x01 \x01(\x0c\x12\x13\n\x0bmanageUsers\x18\x02 \x01(\x08\x12\x15\n\rmanageRecords\x18\x03 \x01(\x08\x12\x0f\n\x07\x63\x61nEdit\x18\x04 \x01(\x08\x12\x10\n\x08\x63\x61nShare\x18\x05 \x01(\x08\"3\n\x18SharedFolderFolderFields\x12\x17\n\x0fsharedFolderUid\x18\x01 \x01(\x0c\"\x8f\x02\n\rFolderRequest\x12\x11\n\tfolderUid\x18\x01 \x01(\x0c\x12&\n\nfolderType\x18\x02 \x01(\x0e\x32\x12.Folder.FolderType\x12\x17\n\x0fparentFolderUid\x18\x03 \x01(\x0c\x12\x12\n\nfolderData\x18\x04 \x01(\x0c\x12\x1a\n\x12\x65ncryptedFolderKey\x18\x05 \x01(\x0c\x12\x36\n\x12sharedFolderFields\x18\x06 \x01(\x0b\x32\x1a.Folder.SharedFolderFields\x12\x42\n\x18sharedFolderFolderFields\x18\x07 \x01(\x0b\x32 .Folder.SharedFolderFolderFields\"E\n\x0e\x46olderResponse\x12\x11\n\tfolderUid\x18\x01 \x01(\x0c\x12\x10\n\x08revision\x18\x02 \x01(\x03\x12\x0e\n\x06status\x18\x03 \x01(\t\"w\n\x19ImportFolderRecordRequest\x12,\n\rfolderRequest\x18\x01 \x03(\x0b\x32\x15.Folder.FolderRequest\x12,\n\rrecordRequest\x18\x02 \x03(\x0b\x32\x15.Folder.RecordRequest\"|\n\x1aImportFolderRecordResponse\x12.\n\x0e\x66olderResponse\x18\x01 \x03(\x0b\x32\x16.Folder.FolderResponse\x12.\n\x0erecordResponse\x18\x02 \x03(\x0b\x32\x16.Folder.RecordResponse*\x1a\n\nRecordType\x12\x0c\n\x08password\x10\x00*^\n\nFolderType\x12\x12\n\x0e\x64\x65\x66\x61ult_folder\x10\x00\x12\x0f\n\x0buser_folder\x10\x01\x12\x11\n\rshared_folder\x10\x02\x12\x18\n\x14shared_folder_folder\x10\x03*V\n\x10\x45ncryptedKeyType\x12\n\n\x06no_key\x10\x00\x12\x19\n\x15\x65ncrypted_by_data_key\x10\x01\x12\x1b\n\x17\x65ncrypted_by_public_key\x10\x02\x42\"\n\x18\x63om.keepersecurity.protoB\x06\x46olderb\x06proto3'
)

_RECORDTYPE = _descriptor.EnumDescriptor(
  name='RecordType',
  full_name='Folder.RecordType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='password', index=0, number=0,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1673,
  serialized_end=1699,
)
_sym_db.RegisterEnumDescriptor(_RECORDTYPE)

RecordType = enum_type_wrapper.EnumTypeWrapper(_RECORDTYPE)
_FOLDERTYPE = _descriptor.EnumDescriptor(
  name='FolderType',
  full_name='Folder.FolderType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='default_folder', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='user_folder', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='shared_folder', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='shared_folder_folder', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1701,
  serialized_end=1795,
)
_sym_db.RegisterEnumDescriptor(_FOLDERTYPE)

FolderType = enum_type_wrapper.EnumTypeWrapper(_FOLDERTYPE)
_ENCRYPTEDKEYTYPE = _descriptor.EnumDescriptor(
  name='EncryptedKeyType',
  full_name='Folder.EncryptedKeyType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='no_key', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='encrypted_by_data_key', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='encrypted_by_public_key', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1797,
  serialized_end=1883,
)
_sym_db.RegisterEnumDescriptor(_ENCRYPTEDKEYTYPE)

EncryptedKeyType = enum_type_wrapper.EnumTypeWrapper(_ENCRYPTEDKEYTYPE)
password = 0
default_folder = 0
user_folder = 1
shared_folder = 2
shared_folder_folder = 3
no_key = 0
encrypted_by_data_key = 1
encrypted_by_public_key = 2



_ENCRYPTEDDATAKEY = _descriptor.Descriptor(
  name='EncryptedDataKey',
  full_name='Folder.EncryptedDataKey',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='encryptedKey', full_name='Folder.EncryptedDataKey.encryptedKey', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedKeyType', full_name='Folder.EncryptedDataKey.encryptedKeyType', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=24,
  serialized_end=116,
)


_SHAREDFOLDERRECORDDATA = _descriptor.Descriptor(
  name='SharedFolderRecordData',
  full_name='Folder.SharedFolderRecordData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='folderUid', full_name='Folder.SharedFolderRecordData.folderUid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recordUid', full_name='Folder.SharedFolderRecordData.recordUid', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='userId', full_name='Folder.SharedFolderRecordData.userId', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedDataKey', full_name='Folder.SharedFolderRecordData.encryptedDataKey', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=119,
  serialized_end=249,
)


_SHAREDFOLDERRECORDDATALIST = _descriptor.Descriptor(
  name='SharedFolderRecordDataList',
  full_name='Folder.SharedFolderRecordDataList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sharedFolderRecordData', full_name='Folder.SharedFolderRecordDataList.sharedFolderRecordData', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=251,
  serialized_end=343,
)


_SHAREDFOLDERRECORDFIX = _descriptor.Descriptor(
  name='SharedFolderRecordFix',
  full_name='Folder.SharedFolderRecordFix',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='folderUid', full_name='Folder.SharedFolderRecordFix.folderUid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recordUid', full_name='Folder.SharedFolderRecordFix.recordUid', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedRecordFolderKey', full_name='Folder.SharedFolderRecordFix.encryptedRecordFolderKey', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=345,
  serialized_end=440,
)


_SHAREDFOLDERRECORDFIXLIST = _descriptor.Descriptor(
  name='SharedFolderRecordFixList',
  full_name='Folder.SharedFolderRecordFixList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sharedFolderRecordFix', full_name='Folder.SharedFolderRecordFixList.sharedFolderRecordFix', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=442,
  serialized_end=531,
)


_RECORDREQUEST = _descriptor.Descriptor(
  name='RecordRequest',
  full_name='Folder.RecordRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='recordUid', full_name='Folder.RecordRequest.recordUid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recordType', full_name='Folder.RecordRequest.recordType', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recordData', full_name='Folder.RecordRequest.recordData', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedRecordKey', full_name='Folder.RecordRequest.encryptedRecordKey', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='folderType', full_name='Folder.RecordRequest.folderType', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='howLongAgo', full_name='Folder.RecordRequest.howLongAgo', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='folderUid', full_name='Folder.RecordRequest.folderUid', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedRecordFolderKey', full_name='Folder.RecordRequest.encryptedRecordFolderKey', index=7,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='extra', full_name='Folder.RecordRequest.extra', index=8,
      number=9, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nonSharedData', full_name='Folder.RecordRequest.nonSharedData', index=9,
      number=10, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fileIds', full_name='Folder.RecordRequest.fileIds', index=10,
      number=11, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=534,
  serialized_end=824,
)


_RECORDRESPONSE = _descriptor.Descriptor(
  name='RecordResponse',
  full_name='Folder.RecordResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='recordUid', full_name='Folder.RecordResponse.recordUid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='revision', full_name='Folder.RecordResponse.revision', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='Folder.RecordResponse.status', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=826,
  serialized_end=895,
)


_SHAREDFOLDERFIELDS = _descriptor.Descriptor(
  name='SharedFolderFields',
  full_name='Folder.SharedFolderFields',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='encryptedFolderName', full_name='Folder.SharedFolderFields.encryptedFolderName', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='manageUsers', full_name='Folder.SharedFolderFields.manageUsers', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='manageRecords', full_name='Folder.SharedFolderFields.manageRecords', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='canEdit', full_name='Folder.SharedFolderFields.canEdit', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='canShare', full_name='Folder.SharedFolderFields.canShare', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=898,
  serialized_end=1026,
)


_SHAREDFOLDERFOLDERFIELDS = _descriptor.Descriptor(
  name='SharedFolderFolderFields',
  full_name='Folder.SharedFolderFolderFields',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sharedFolderUid', full_name='Folder.SharedFolderFolderFields.sharedFolderUid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1028,
  serialized_end=1079,
)


_FOLDERREQUEST = _descriptor.Descriptor(
  name='FolderRequest',
  full_name='Folder.FolderRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='folderUid', full_name='Folder.FolderRequest.folderUid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='folderType', full_name='Folder.FolderRequest.folderType', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='parentFolderUid', full_name='Folder.FolderRequest.parentFolderUid', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='folderData', full_name='Folder.FolderRequest.folderData', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedFolderKey', full_name='Folder.FolderRequest.encryptedFolderKey', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sharedFolderFields', full_name='Folder.FolderRequest.sharedFolderFields', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sharedFolderFolderFields', full_name='Folder.FolderRequest.sharedFolderFolderFields', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1082,
  serialized_end=1353,
)


_FOLDERRESPONSE = _descriptor.Descriptor(
  name='FolderResponse',
  full_name='Folder.FolderResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='folderUid', full_name='Folder.FolderResponse.folderUid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='revision', full_name='Folder.FolderResponse.revision', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='Folder.FolderResponse.status', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1355,
  serialized_end=1424,
)


_IMPORTFOLDERRECORDREQUEST = _descriptor.Descriptor(
  name='ImportFolderRecordRequest',
  full_name='Folder.ImportFolderRecordRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='folderRequest', full_name='Folder.ImportFolderRecordRequest.folderRequest', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recordRequest', full_name='Folder.ImportFolderRecordRequest.recordRequest', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1426,
  serialized_end=1545,
)


_IMPORTFOLDERRECORDRESPONSE = _descriptor.Descriptor(
  name='ImportFolderRecordResponse',
  full_name='Folder.ImportFolderRecordResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='folderResponse', full_name='Folder.ImportFolderRecordResponse.folderResponse', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recordResponse', full_name='Folder.ImportFolderRecordResponse.recordResponse', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1547,
  serialized_end=1671,
)

_ENCRYPTEDDATAKEY.fields_by_name['encryptedKeyType'].enum_type = _ENCRYPTEDKEYTYPE
_SHAREDFOLDERRECORDDATA.fields_by_name['encryptedDataKey'].message_type = _ENCRYPTEDDATAKEY
_SHAREDFOLDERRECORDDATALIST.fields_by_name['sharedFolderRecordData'].message_type = _SHAREDFOLDERRECORDDATA
_SHAREDFOLDERRECORDFIXLIST.fields_by_name['sharedFolderRecordFix'].message_type = _SHAREDFOLDERRECORDFIX
_RECORDREQUEST.fields_by_name['recordType'].enum_type = _RECORDTYPE
_RECORDREQUEST.fields_by_name['folderType'].enum_type = _FOLDERTYPE
_FOLDERREQUEST.fields_by_name['folderType'].enum_type = _FOLDERTYPE
_FOLDERREQUEST.fields_by_name['sharedFolderFields'].message_type = _SHAREDFOLDERFIELDS
_FOLDERREQUEST.fields_by_name['sharedFolderFolderFields'].message_type = _SHAREDFOLDERFOLDERFIELDS
_IMPORTFOLDERRECORDREQUEST.fields_by_name['folderRequest'].message_type = _FOLDERREQUEST
_IMPORTFOLDERRECORDREQUEST.fields_by_name['recordRequest'].message_type = _RECORDREQUEST
_IMPORTFOLDERRECORDRESPONSE.fields_by_name['folderResponse'].message_type = _FOLDERRESPONSE
_IMPORTFOLDERRECORDRESPONSE.fields_by_name['recordResponse'].message_type = _RECORDRESPONSE
DESCRIPTOR.message_types_by_name['EncryptedDataKey'] = _ENCRYPTEDDATAKEY
DESCRIPTOR.message_types_by_name['SharedFolderRecordData'] = _SHAREDFOLDERRECORDDATA
DESCRIPTOR.message_types_by_name['SharedFolderRecordDataList'] = _SHAREDFOLDERRECORDDATALIST
DESCRIPTOR.message_types_by_name['SharedFolderRecordFix'] = _SHAREDFOLDERRECORDFIX
DESCRIPTOR.message_types_by_name['SharedFolderRecordFixList'] = _SHAREDFOLDERRECORDFIXLIST
DESCRIPTOR.message_types_by_name['RecordRequest'] = _RECORDREQUEST
DESCRIPTOR.message_types_by_name['RecordResponse'] = _RECORDRESPONSE
DESCRIPTOR.message_types_by_name['SharedFolderFields'] = _SHAREDFOLDERFIELDS
DESCRIPTOR.message_types_by_name['SharedFolderFolderFields'] = _SHAREDFOLDERFOLDERFIELDS
DESCRIPTOR.message_types_by_name['FolderRequest'] = _FOLDERREQUEST
DESCRIPTOR.message_types_by_name['FolderResponse'] = _FOLDERRESPONSE
DESCRIPTOR.message_types_by_name['ImportFolderRecordRequest'] = _IMPORTFOLDERRECORDREQUEST
DESCRIPTOR.message_types_by_name['ImportFolderRecordResponse'] = _IMPORTFOLDERRECORDRESPONSE
DESCRIPTOR.enum_types_by_name['RecordType'] = _RECORDTYPE
DESCRIPTOR.enum_types_by_name['FolderType'] = _FOLDERTYPE
DESCRIPTOR.enum_types_by_name['EncryptedKeyType'] = _ENCRYPTEDKEYTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EncryptedDataKey = _reflection.GeneratedProtocolMessageType('EncryptedDataKey', (_message.Message,), {
  'DESCRIPTOR' : _ENCRYPTEDDATAKEY,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.EncryptedDataKey)
  })
_sym_db.RegisterMessage(EncryptedDataKey)

SharedFolderRecordData = _reflection.GeneratedProtocolMessageType('SharedFolderRecordData', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERRECORDDATA,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderRecordData)
  })
_sym_db.RegisterMessage(SharedFolderRecordData)

SharedFolderRecordDataList = _reflection.GeneratedProtocolMessageType('SharedFolderRecordDataList', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERRECORDDATALIST,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderRecordDataList)
  })
_sym_db.RegisterMessage(SharedFolderRecordDataList)

SharedFolderRecordFix = _reflection.GeneratedProtocolMessageType('SharedFolderRecordFix', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERRECORDFIX,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderRecordFix)
  })
_sym_db.RegisterMessage(SharedFolderRecordFix)

SharedFolderRecordFixList = _reflection.GeneratedProtocolMessageType('SharedFolderRecordFixList', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERRECORDFIXLIST,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderRecordFixList)
  })
_sym_db.RegisterMessage(SharedFolderRecordFixList)

RecordRequest = _reflection.GeneratedProtocolMessageType('RecordRequest', (_message.Message,), {
  'DESCRIPTOR' : _RECORDREQUEST,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.RecordRequest)
  })
_sym_db.RegisterMessage(RecordRequest)

RecordResponse = _reflection.GeneratedProtocolMessageType('RecordResponse', (_message.Message,), {
  'DESCRIPTOR' : _RECORDRESPONSE,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.RecordResponse)
  })
_sym_db.RegisterMessage(RecordResponse)

SharedFolderFields = _reflection.GeneratedProtocolMessageType('SharedFolderFields', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERFIELDS,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderFields)
  })
_sym_db.RegisterMessage(SharedFolderFields)

SharedFolderFolderFields = _reflection.GeneratedProtocolMessageType('SharedFolderFolderFields', (_message.Message,), {
  'DESCRIPTOR' : _SHAREDFOLDERFOLDERFIELDS,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.SharedFolderFolderFields)
  })
_sym_db.RegisterMessage(SharedFolderFolderFields)

FolderRequest = _reflection.GeneratedProtocolMessageType('FolderRequest', (_message.Message,), {
  'DESCRIPTOR' : _FOLDERREQUEST,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.FolderRequest)
  })
_sym_db.RegisterMessage(FolderRequest)

FolderResponse = _reflection.GeneratedProtocolMessageType('FolderResponse', (_message.Message,), {
  'DESCRIPTOR' : _FOLDERRESPONSE,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.FolderResponse)
  })
_sym_db.RegisterMessage(FolderResponse)

ImportFolderRecordRequest = _reflection.GeneratedProtocolMessageType('ImportFolderRecordRequest', (_message.Message,), {
  'DESCRIPTOR' : _IMPORTFOLDERRECORDREQUEST,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.ImportFolderRecordRequest)
  })
_sym_db.RegisterMessage(ImportFolderRecordRequest)

ImportFolderRecordResponse = _reflection.GeneratedProtocolMessageType('ImportFolderRecordResponse', (_message.Message,), {
  'DESCRIPTOR' : _IMPORTFOLDERRECORDRESPONSE,
  '__module__' : 'folder_pb2'
  # @@protoc_insertion_point(class_scope:Folder.ImportFolderRecordResponse)
  })
_sym_db.RegisterMessage(ImportFolderRecordResponse)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
