# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: APIRequest.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='APIRequest.proto',
  package='Authentication',
  syntax='proto3',
  serialized_options=_b('\n\030com.keepersecurity.protoB\016Authentication'),
  serialized_pb=_b('\n\x10\x41PIRequest.proto\x12\x0e\x41uthentication\"m\n\nApiRequest\x12 \n\x18\x65ncryptedTransmissionKey\x18\x01 \x01(\x0c\x12\x13\n\x0bpublicKeyId\x18\x02 \x01(\x05\x12\x0e\n\x06locale\x18\x03 \x01(\t\x12\x18\n\x10\x65ncryptedPayload\x18\x04 \x01(\x0c\"j\n\x11\x41piRequestPayload\x12\x0f\n\x07payload\x18\x01 \x01(\x0c\x12\x1d\n\x15\x65ncryptedSessionToken\x18\x02 \x01(\x0c\x12\x11\n\ttimeToken\x18\x03 \x01(\x0c\x12\x12\n\napiVersion\x18\x04 \x01(\x05\"6\n\tTransform\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\x1c\n\x14\x65ncryptedDeviceToken\x18\x02 \x01(\x0c\":\n\rDeviceRequest\x12\x15\n\rclientVersion\x18\x01 \x01(\t\x12\x12\n\ndeviceName\x18\x02 \x01(\t\"T\n\x0b\x41uthRequest\x12\x15\n\rclientVersion\x18\x01 \x01(\t\x12\x10\n\x08username\x18\x02 \x01(\t\x12\x1c\n\x14\x65ncryptedDeviceToken\x18\x03 \x01(\x0c\"o\n\x14NewUserMinimumParams\x12\x19\n\x11minimumIterations\x18\x01 \x01(\x05\x12\x1a\n\x12passwordMatchRegex\x18\x02 \x03(\t\x12 \n\x18passwordMatchDescription\x18\x03 \x03(\t\"\x89\x01\n\x0fPreLoginRequest\x12\x30\n\x0b\x61uthRequest\x18\x01 \x01(\x0b\x32\x1b.Authentication.AuthRequest\x12,\n\tloginType\x18\x02 \x01(\x0e\x32\x19.Authentication.LoginType\x12\x16\n\x0etwoFactorToken\x18\x03 \x01(\x0c\"\xea\x01\n\x0cLoginRequest\x12\x30\n\x0b\x61uthRequest\x18\x01 \x01(\x0b\x32\x1b.Authentication.AuthRequest\x12,\n\tloginType\x18\x02 \x01(\x0e\x32\x19.Authentication.LoginType\x12\x1f\n\x17\x61uthenticationHashPrime\x18\x03 \x01(\x0c\x12\x15\n\rrandomHashKey\x18\x04 \x01(\x0c\x12\x1f\n\x17\x65ncryptedTwoFactorToken\x18\x05 \x01(\x0c\x12!\n\x19\x65ncryptedBreachWatchToken\x18\x06 \x01(\x0c\"\xf8\x02\n\x13RegistrationRequest\x12\x30\n\x0b\x61uthRequest\x18\x01 \x01(\x0b\x32\x1b.Authentication.AuthRequest\x12\x38\n\x0fuserAuthRequest\x18\x02 \x01(\x0b\x32\x1f.Authentication.UserAuthRequest\x12\x1a\n\x12\x65ncryptedClientKey\x18\x03 \x01(\x0c\x12\x1b\n\x13\x65ncryptedPrivateKey\x18\x04 \x01(\x0c\x12\x11\n\tpublicKey\x18\x05 \x01(\x0c\x12\x18\n\x10verificationCode\x18\x06 \x01(\t\x12\x1e\n\x16\x64\x65precatedAuthHashHash\x18\x07 \x01(\x0c\x12$\n\x1c\x64\x65precatedEncryptedClientKey\x18\x08 \x01(\x0c\x12%\n\x1d\x64\x65precatedEncryptedPrivateKey\x18\t \x01(\x0c\x12\"\n\x1a\x64\x65precatedEncryptionParams\x18\n \x01(\x0c\"\\\n\x0e\x44\x65viceResponse\x12\x1c\n\x14\x65ncryptedDeviceToken\x18\x01 \x01(\x0c\x12,\n\x06status\x18\x02 \x01(\x0e\x32\x1c.Authentication.DeviceStatus\"V\n\x04Salt\x12\x12\n\niterations\x18\x01 \x01(\x05\x12\x0c\n\x04salt\x18\x02 \x01(\x0c\x12\x11\n\talgorithm\x18\x03 \x01(\x05\x12\x0b\n\x03uid\x18\x04 \x01(\x0c\x12\x0c\n\x04name\x18\x05 \x01(\t\" \n\x10TwoFactorChannel\x12\x0c\n\x04type\x18\x01 \x01(\x05\"\xa0\x01\n\x10PreLoginResponse\x12,\n\x06status\x18\x01 \x01(\x0e\x32\x1c.Authentication.DeviceStatus\x12\"\n\x04salt\x18\x02 \x03(\x0b\x32\x14.Authentication.Salt\x12:\n\x10twoFactorChannel\x18\x03 \x03(\x0b\x32 .Authentication.TwoFactorChannel\"\xa0\x03\n\rLoginResponse\x12\x1e\n\x16\x65ncryrptedSessionToken\x18\x01 \x01(\x0c\x12&\n\x05vault\x18\x02 \x01(\x0b\x32\x17.Authentication.License\x12%\n\x04\x63hat\x18\x03 \x01(\x0b\x32\x17.Authentication.License\x12(\n\x07storage\x18\x04 \x01(\x0b\x32\x17.Authentication.License\x12,\n\x0b\x62reachWatch\x18\x05 \x01(\x0b\x32\x17.Authentication.License\x12\x30\n\x0b\x61\x63\x63ountType\x18\x06 \x01(\x0e\x32\x1b.Authentication.AccountType\x12\x14\n\x0c\x65ncryptedDAT\x18\x07 \x01(\x0c\x12\x14\n\x0c\x65ncryptedPAT\x18\x08 \x01(\x0c\x12\x14\n\x0c\x65ncryptedEAT\x18\t \x01(\x0c\x12\x18\n\x10\x65ncryptedDataKey\x18\n \x01(\x0c\x12:\n\x10sessionTokenType\x18\x0b \x03(\x0e\x32 .Authentication.SessionTokenType\"\x83\x01\n\x07License\x12\x0f\n\x07\x63reated\x18\x01 \x01(\x03\x12\x12\n\nexpiration\x18\x02 \x01(\x03\x12\x34\n\rlicenseStatus\x18\x03 \x01(\x0e\x32\x1d.Authentication.LicenseStatus\x12\x0c\n\x04paid\x18\x04 \x01(\x08\x12\x0f\n\x07message\x18\x05 \x01(\t\"G\n\x0fOwnerlessRecord\x12\x11\n\trecordUid\x18\x01 \x01(\x0c\x12\x11\n\trecordKey\x18\x02 \x01(\x0c\x12\x0e\n\x06status\x18\x03 \x01(\x05\"L\n\x10OwnerlessRecords\x12\x38\n\x0fownerlessRecord\x18\x01 \x03(\x0b\x32\x1f.Authentication.OwnerlessRecord\"\xc4\x01\n\x0fUserAuthRequest\x12\x0b\n\x03uid\x18\x01 \x01(\x0c\x12\x0c\n\x04salt\x18\x02 \x01(\x0c\x12\x12\n\niterations\x18\x03 \x01(\x05\x12\x1a\n\x12\x65ncryptedClientKey\x18\x04 \x01(\x0c\x12\x10\n\x08\x61uthHash\x18\x05 \x01(\x0c\x12\x18\n\x10\x65ncryptedDataKey\x18\x06 \x01(\x0c\x12,\n\tloginType\x18\x07 \x01(\x0e\x32\x19.Authentication.LoginType\x12\x0c\n\x04name\x18\x08 \x01(\t\"\x19\n\nUidRequest\x12\x0b\n\x03uid\x18\x01 \x03(\x0c\"W\n DeviceClientVersionUpdateRequest\x12\x1c\n\x14\x65ncryptedDeviceToken\x18\x01 \x01(\x0c\x12\x15\n\rclientVersion\x18\x02 \x01(\t\"\xd0\x01\n\x16\x43onvertUserToV3Request\x12\x30\n\x0b\x61uthRequest\x18\x01 \x01(\x0b\x32\x1b.Authentication.AuthRequest\x12\x38\n\x0fuserAuthRequest\x18\x02 \x01(\x0b\x32\x1f.Authentication.UserAuthRequest\x12\x1a\n\x12\x65ncryptedClientKey\x18\x03 \x01(\x0c\x12\x1b\n\x13\x65ncryptedPrivateKey\x18\x04 \x01(\x0c\x12\x11\n\tpublicKey\x18\x05 \x01(\x0c\"$\n\x10RevisionResponse\x12\x10\n\x08revision\x18\x01 \x01(\x03*\xb9\x02\n\x11SupportedLanguage\x12\x0b\n\x07\x45NGLISH\x10\x00\x12\n\n\x06\x41RABIC\x10\x01\x12\x0b\n\x07\x42RITISH\x10\x02\x12\x0b\n\x07\x43HINESE\x10\x03\x12\x15\n\x11\x43HINESE_HONG_KONG\x10\x04\x12\x12\n\x0e\x43HINESE_TAIWAN\x10\x05\x12\t\n\x05\x44UTCH\x10\x06\x12\n\n\x06\x46RENCH\x10\x07\x12\n\n\x06GERMAN\x10\x08\x12\t\n\x05GREEK\x10\t\x12\n\n\x06HEBREW\x10\n\x12\x0b\n\x07ITALIAN\x10\x0b\x12\x0c\n\x08JAPANESE\x10\x0c\x12\n\n\x06KOREAN\x10\r\x12\n\n\x06POLISH\x10\x0e\x12\x0e\n\nPORTUGUESE\x10\x0f\x12\x15\n\x11PORTUGUESE_BRAZIL\x10\x10\x12\x0c\n\x08ROMANIAN\x10\x11\x12\x0b\n\x07RUSSIAN\x10\x12\x12\n\n\x06SLOVAK\x10\x13\x12\x0b\n\x07SPANISH\x10\x14*E\n\tLoginType\x12\n\n\x06NORMAL\x10\x00\x12\x07\n\x03SSO\x10\x01\x12\x07\n\x03\x42IO\x10\x02\x12\r\n\tALTERNATE\x10\x03\x12\x0b\n\x07OFFLINE\x10\x04*>\n\x0c\x44\x65viceStatus\x12\x11\n\rNEED_APPROVAL\x10\x00\x12\x06\n\x02OK\x10\x01\x12\x13\n\x0f\x44\x45VICE_DISABLED\x10\x02*A\n\rLicenseStatus\x12\t\n\x05OTHER\x10\x00\x12\n\n\x06\x41\x43TIVE\x10\x01\x12\x0b\n\x07\x45XPIRED\x10\x02\x12\x0c\n\x08\x44ISABLED\x10\x03*7\n\x0b\x41\x63\x63ountType\x12\x0c\n\x08\x43ONSUMER\x10\x00\x12\n\n\x06\x46\x41MILY\x10\x01\x12\x0e\n\nENTERPRISE\x10\x02*]\n\x10SessionTokenType\x12\x12\n\x0eNO_RESTRICTION\x10\x00\x12\x14\n\x10\x41\x43\x43OUNT_RECOVERY\x10\x01\x12\x11\n\rSHARE_ACCOUNT\x10\x02\x12\x0c\n\x08PURCHASE\x10\x03\x42*\n\x18\x63om.keepersecurity.protoB\x0e\x41uthenticationb\x06proto3')
)

_SUPPORTEDLANGUAGE = _descriptor.EnumDescriptor(
  name='SupportedLanguage',
  full_name='Authentication.SupportedLanguage',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ENGLISH', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ARABIC', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BRITISH', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CHINESE', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CHINESE_HONG_KONG', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CHINESE_TAIWAN', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DUTCH', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FRENCH', index=7, number=7,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GERMAN', index=8, number=8,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='GREEK', index=9, number=9,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HEBREW', index=10, number=10,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ITALIAN', index=11, number=11,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='JAPANESE', index=12, number=12,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='KOREAN', index=13, number=13,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='POLISH', index=14, number=14,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PORTUGUESE', index=15, number=15,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PORTUGUESE_BRAZIL', index=16, number=16,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ROMANIAN', index=17, number=17,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RUSSIAN', index=18, number=18,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SLOVAK', index=19, number=19,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SPANISH', index=20, number=20,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2974,
  serialized_end=3287,
)
_sym_db.RegisterEnumDescriptor(_SUPPORTEDLANGUAGE)

SupportedLanguage = enum_type_wrapper.EnumTypeWrapper(_SUPPORTEDLANGUAGE)
_LOGINTYPE = _descriptor.EnumDescriptor(
  name='LoginType',
  full_name='Authentication.LoginType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NORMAL', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SSO', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='BIO', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ALTERNATE', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OFFLINE', index=4, number=4,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3289,
  serialized_end=3358,
)
_sym_db.RegisterEnumDescriptor(_LOGINTYPE)

LoginType = enum_type_wrapper.EnumTypeWrapper(_LOGINTYPE)
_DEVICESTATUS = _descriptor.EnumDescriptor(
  name='DeviceStatus',
  full_name='Authentication.DeviceStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NEED_APPROVAL', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OK', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DEVICE_DISABLED', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3360,
  serialized_end=3422,
)
_sym_db.RegisterEnumDescriptor(_DEVICESTATUS)

DeviceStatus = enum_type_wrapper.EnumTypeWrapper(_DEVICESTATUS)
_LICENSESTATUS = _descriptor.EnumDescriptor(
  name='LicenseStatus',
  full_name='Authentication.LicenseStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OTHER', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ACTIVE', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='EXPIRED', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DISABLED', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3424,
  serialized_end=3489,
)
_sym_db.RegisterEnumDescriptor(_LICENSESTATUS)

LicenseStatus = enum_type_wrapper.EnumTypeWrapper(_LICENSESTATUS)
_ACCOUNTTYPE = _descriptor.EnumDescriptor(
  name='AccountType',
  full_name='Authentication.AccountType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CONSUMER', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FAMILY', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ENTERPRISE', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3491,
  serialized_end=3546,
)
_sym_db.RegisterEnumDescriptor(_ACCOUNTTYPE)

AccountType = enum_type_wrapper.EnumTypeWrapper(_ACCOUNTTYPE)
_SESSIONTOKENTYPE = _descriptor.EnumDescriptor(
  name='SessionTokenType',
  full_name='Authentication.SessionTokenType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='NO_RESTRICTION', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ACCOUNT_RECOVERY', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SHARE_ACCOUNT', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PURCHASE', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=3548,
  serialized_end=3641,
)
_sym_db.RegisterEnumDescriptor(_SESSIONTOKENTYPE)

SessionTokenType = enum_type_wrapper.EnumTypeWrapper(_SESSIONTOKENTYPE)
ENGLISH = 0
ARABIC = 1
BRITISH = 2
CHINESE = 3
CHINESE_HONG_KONG = 4
CHINESE_TAIWAN = 5
DUTCH = 6
FRENCH = 7
GERMAN = 8
GREEK = 9
HEBREW = 10
ITALIAN = 11
JAPANESE = 12
KOREAN = 13
POLISH = 14
PORTUGUESE = 15
PORTUGUESE_BRAZIL = 16
ROMANIAN = 17
RUSSIAN = 18
SLOVAK = 19
SPANISH = 20
NORMAL = 0
SSO = 1
BIO = 2
ALTERNATE = 3
OFFLINE = 4
NEED_APPROVAL = 0
OK = 1
DEVICE_DISABLED = 2
OTHER = 0
ACTIVE = 1
EXPIRED = 2
DISABLED = 3
CONSUMER = 0
FAMILY = 1
ENTERPRISE = 2
NO_RESTRICTION = 0
ACCOUNT_RECOVERY = 1
SHARE_ACCOUNT = 2
PURCHASE = 3



_APIREQUEST = _descriptor.Descriptor(
  name='ApiRequest',
  full_name='Authentication.ApiRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='encryptedTransmissionKey', full_name='Authentication.ApiRequest.encryptedTransmissionKey', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='publicKeyId', full_name='Authentication.ApiRequest.publicKeyId', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='locale', full_name='Authentication.ApiRequest.locale', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedPayload', full_name='Authentication.ApiRequest.encryptedPayload', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=36,
  serialized_end=145,
)


_APIREQUESTPAYLOAD = _descriptor.Descriptor(
  name='ApiRequestPayload',
  full_name='Authentication.ApiRequestPayload',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='payload', full_name='Authentication.ApiRequestPayload.payload', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedSessionToken', full_name='Authentication.ApiRequestPayload.encryptedSessionToken', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timeToken', full_name='Authentication.ApiRequestPayload.timeToken', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='apiVersion', full_name='Authentication.ApiRequestPayload.apiVersion', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=147,
  serialized_end=253,
)


_TRANSFORM = _descriptor.Descriptor(
  name='Transform',
  full_name='Authentication.Transform',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='Authentication.Transform.key', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedDeviceToken', full_name='Authentication.Transform.encryptedDeviceToken', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=255,
  serialized_end=309,
)


_DEVICEREQUEST = _descriptor.Descriptor(
  name='DeviceRequest',
  full_name='Authentication.DeviceRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='clientVersion', full_name='Authentication.DeviceRequest.clientVersion', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deviceName', full_name='Authentication.DeviceRequest.deviceName', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=311,
  serialized_end=369,
)


_AUTHREQUEST = _descriptor.Descriptor(
  name='AuthRequest',
  full_name='Authentication.AuthRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='clientVersion', full_name='Authentication.AuthRequest.clientVersion', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='username', full_name='Authentication.AuthRequest.username', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedDeviceToken', full_name='Authentication.AuthRequest.encryptedDeviceToken', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=371,
  serialized_end=455,
)


_NEWUSERMINIMUMPARAMS = _descriptor.Descriptor(
  name='NewUserMinimumParams',
  full_name='Authentication.NewUserMinimumParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='minimumIterations', full_name='Authentication.NewUserMinimumParams.minimumIterations', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='passwordMatchRegex', full_name='Authentication.NewUserMinimumParams.passwordMatchRegex', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='passwordMatchDescription', full_name='Authentication.NewUserMinimumParams.passwordMatchDescription', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=457,
  serialized_end=568,
)


_PRELOGINREQUEST = _descriptor.Descriptor(
  name='PreLoginRequest',
  full_name='Authentication.PreLoginRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='authRequest', full_name='Authentication.PreLoginRequest.authRequest', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='loginType', full_name='Authentication.PreLoginRequest.loginType', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='twoFactorToken', full_name='Authentication.PreLoginRequest.twoFactorToken', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=571,
  serialized_end=708,
)


_LOGINREQUEST = _descriptor.Descriptor(
  name='LoginRequest',
  full_name='Authentication.LoginRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='authRequest', full_name='Authentication.LoginRequest.authRequest', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='loginType', full_name='Authentication.LoginRequest.loginType', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='authenticationHashPrime', full_name='Authentication.LoginRequest.authenticationHashPrime', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='randomHashKey', full_name='Authentication.LoginRequest.randomHashKey', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedTwoFactorToken', full_name='Authentication.LoginRequest.encryptedTwoFactorToken', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedBreachWatchToken', full_name='Authentication.LoginRequest.encryptedBreachWatchToken', index=5,
      number=6, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=711,
  serialized_end=945,
)


_REGISTRATIONREQUEST = _descriptor.Descriptor(
  name='RegistrationRequest',
  full_name='Authentication.RegistrationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='authRequest', full_name='Authentication.RegistrationRequest.authRequest', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='userAuthRequest', full_name='Authentication.RegistrationRequest.userAuthRequest', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedClientKey', full_name='Authentication.RegistrationRequest.encryptedClientKey', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedPrivateKey', full_name='Authentication.RegistrationRequest.encryptedPrivateKey', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='publicKey', full_name='Authentication.RegistrationRequest.publicKey', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='verificationCode', full_name='Authentication.RegistrationRequest.verificationCode', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deprecatedAuthHashHash', full_name='Authentication.RegistrationRequest.deprecatedAuthHashHash', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deprecatedEncryptedClientKey', full_name='Authentication.RegistrationRequest.deprecatedEncryptedClientKey', index=7,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deprecatedEncryptedPrivateKey', full_name='Authentication.RegistrationRequest.deprecatedEncryptedPrivateKey', index=8,
      number=9, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='deprecatedEncryptionParams', full_name='Authentication.RegistrationRequest.deprecatedEncryptionParams', index=9,
      number=10, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=948,
  serialized_end=1324,
)


_DEVICERESPONSE = _descriptor.Descriptor(
  name='DeviceResponse',
  full_name='Authentication.DeviceResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='encryptedDeviceToken', full_name='Authentication.DeviceResponse.encryptedDeviceToken', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='Authentication.DeviceResponse.status', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1326,
  serialized_end=1418,
)


_SALT = _descriptor.Descriptor(
  name='Salt',
  full_name='Authentication.Salt',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='iterations', full_name='Authentication.Salt.iterations', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='salt', full_name='Authentication.Salt.salt', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='algorithm', full_name='Authentication.Salt.algorithm', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uid', full_name='Authentication.Salt.uid', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='Authentication.Salt.name', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1420,
  serialized_end=1506,
)


_TWOFACTORCHANNEL = _descriptor.Descriptor(
  name='TwoFactorChannel',
  full_name='Authentication.TwoFactorChannel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='Authentication.TwoFactorChannel.type', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1508,
  serialized_end=1540,
)


_PRELOGINRESPONSE = _descriptor.Descriptor(
  name='PreLoginResponse',
  full_name='Authentication.PreLoginResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='Authentication.PreLoginResponse.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='salt', full_name='Authentication.PreLoginResponse.salt', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='twoFactorChannel', full_name='Authentication.PreLoginResponse.twoFactorChannel', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1543,
  serialized_end=1703,
)


_LOGINRESPONSE = _descriptor.Descriptor(
  name='LoginResponse',
  full_name='Authentication.LoginResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='encryrptedSessionToken', full_name='Authentication.LoginResponse.encryrptedSessionToken', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vault', full_name='Authentication.LoginResponse.vault', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='chat', full_name='Authentication.LoginResponse.chat', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='storage', full_name='Authentication.LoginResponse.storage', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='breachWatch', full_name='Authentication.LoginResponse.breachWatch', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='accountType', full_name='Authentication.LoginResponse.accountType', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedDAT', full_name='Authentication.LoginResponse.encryptedDAT', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedPAT', full_name='Authentication.LoginResponse.encryptedPAT', index=7,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedEAT', full_name='Authentication.LoginResponse.encryptedEAT', index=8,
      number=9, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedDataKey', full_name='Authentication.LoginResponse.encryptedDataKey', index=9,
      number=10, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sessionTokenType', full_name='Authentication.LoginResponse.sessionTokenType', index=10,
      number=11, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1706,
  serialized_end=2122,
)


_LICENSE = _descriptor.Descriptor(
  name='License',
  full_name='Authentication.License',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='created', full_name='Authentication.License.created', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='expiration', full_name='Authentication.License.expiration', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='licenseStatus', full_name='Authentication.License.licenseStatus', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='paid', full_name='Authentication.License.paid', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message', full_name='Authentication.License.message', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2125,
  serialized_end=2256,
)


_OWNERLESSRECORD = _descriptor.Descriptor(
  name='OwnerlessRecord',
  full_name='Authentication.OwnerlessRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='recordUid', full_name='Authentication.OwnerlessRecord.recordUid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='recordKey', full_name='Authentication.OwnerlessRecord.recordKey', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='Authentication.OwnerlessRecord.status', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2258,
  serialized_end=2329,
)


_OWNERLESSRECORDS = _descriptor.Descriptor(
  name='OwnerlessRecords',
  full_name='Authentication.OwnerlessRecords',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ownerlessRecord', full_name='Authentication.OwnerlessRecords.ownerlessRecord', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2331,
  serialized_end=2407,
)


_USERAUTHREQUEST = _descriptor.Descriptor(
  name='UserAuthRequest',
  full_name='Authentication.UserAuthRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uid', full_name='Authentication.UserAuthRequest.uid', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='salt', full_name='Authentication.UserAuthRequest.salt', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='iterations', full_name='Authentication.UserAuthRequest.iterations', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedClientKey', full_name='Authentication.UserAuthRequest.encryptedClientKey', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='authHash', full_name='Authentication.UserAuthRequest.authHash', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedDataKey', full_name='Authentication.UserAuthRequest.encryptedDataKey', index=5,
      number=6, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='loginType', full_name='Authentication.UserAuthRequest.loginType', index=6,
      number=7, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='name', full_name='Authentication.UserAuthRequest.name', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2410,
  serialized_end=2606,
)


_UIDREQUEST = _descriptor.Descriptor(
  name='UidRequest',
  full_name='Authentication.UidRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uid', full_name='Authentication.UidRequest.uid', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2608,
  serialized_end=2633,
)


_DEVICECLIENTVERSIONUPDATEREQUEST = _descriptor.Descriptor(
  name='DeviceClientVersionUpdateRequest',
  full_name='Authentication.DeviceClientVersionUpdateRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='encryptedDeviceToken', full_name='Authentication.DeviceClientVersionUpdateRequest.encryptedDeviceToken', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='clientVersion', full_name='Authentication.DeviceClientVersionUpdateRequest.clientVersion', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2635,
  serialized_end=2722,
)


_CONVERTUSERTOV3REQUEST = _descriptor.Descriptor(
  name='ConvertUserToV3Request',
  full_name='Authentication.ConvertUserToV3Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='authRequest', full_name='Authentication.ConvertUserToV3Request.authRequest', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='userAuthRequest', full_name='Authentication.ConvertUserToV3Request.userAuthRequest', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedClientKey', full_name='Authentication.ConvertUserToV3Request.encryptedClientKey', index=2,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='encryptedPrivateKey', full_name='Authentication.ConvertUserToV3Request.encryptedPrivateKey', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='publicKey', full_name='Authentication.ConvertUserToV3Request.publicKey', index=4,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2725,
  serialized_end=2933,
)


_REVISIONRESPONSE = _descriptor.Descriptor(
  name='RevisionResponse',
  full_name='Authentication.RevisionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='revision', full_name='Authentication.RevisionResponse.revision', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2935,
  serialized_end=2971,
)

_PRELOGINREQUEST.fields_by_name['authRequest'].message_type = _AUTHREQUEST
_PRELOGINREQUEST.fields_by_name['loginType'].enum_type = _LOGINTYPE
_LOGINREQUEST.fields_by_name['authRequest'].message_type = _AUTHREQUEST
_LOGINREQUEST.fields_by_name['loginType'].enum_type = _LOGINTYPE
_REGISTRATIONREQUEST.fields_by_name['authRequest'].message_type = _AUTHREQUEST
_REGISTRATIONREQUEST.fields_by_name['userAuthRequest'].message_type = _USERAUTHREQUEST
_DEVICERESPONSE.fields_by_name['status'].enum_type = _DEVICESTATUS
_PRELOGINRESPONSE.fields_by_name['status'].enum_type = _DEVICESTATUS
_PRELOGINRESPONSE.fields_by_name['salt'].message_type = _SALT
_PRELOGINRESPONSE.fields_by_name['twoFactorChannel'].message_type = _TWOFACTORCHANNEL
_LOGINRESPONSE.fields_by_name['vault'].message_type = _LICENSE
_LOGINRESPONSE.fields_by_name['chat'].message_type = _LICENSE
_LOGINRESPONSE.fields_by_name['storage'].message_type = _LICENSE
_LOGINRESPONSE.fields_by_name['breachWatch'].message_type = _LICENSE
_LOGINRESPONSE.fields_by_name['accountType'].enum_type = _ACCOUNTTYPE
_LOGINRESPONSE.fields_by_name['sessionTokenType'].enum_type = _SESSIONTOKENTYPE
_LICENSE.fields_by_name['licenseStatus'].enum_type = _LICENSESTATUS
_OWNERLESSRECORDS.fields_by_name['ownerlessRecord'].message_type = _OWNERLESSRECORD
_USERAUTHREQUEST.fields_by_name['loginType'].enum_type = _LOGINTYPE
_CONVERTUSERTOV3REQUEST.fields_by_name['authRequest'].message_type = _AUTHREQUEST
_CONVERTUSERTOV3REQUEST.fields_by_name['userAuthRequest'].message_type = _USERAUTHREQUEST
DESCRIPTOR.message_types_by_name['ApiRequest'] = _APIREQUEST
DESCRIPTOR.message_types_by_name['ApiRequestPayload'] = _APIREQUESTPAYLOAD
DESCRIPTOR.message_types_by_name['Transform'] = _TRANSFORM
DESCRIPTOR.message_types_by_name['DeviceRequest'] = _DEVICEREQUEST
DESCRIPTOR.message_types_by_name['AuthRequest'] = _AUTHREQUEST
DESCRIPTOR.message_types_by_name['NewUserMinimumParams'] = _NEWUSERMINIMUMPARAMS
DESCRIPTOR.message_types_by_name['PreLoginRequest'] = _PRELOGINREQUEST
DESCRIPTOR.message_types_by_name['LoginRequest'] = _LOGINREQUEST
DESCRIPTOR.message_types_by_name['RegistrationRequest'] = _REGISTRATIONREQUEST
DESCRIPTOR.message_types_by_name['DeviceResponse'] = _DEVICERESPONSE
DESCRIPTOR.message_types_by_name['Salt'] = _SALT
DESCRIPTOR.message_types_by_name['TwoFactorChannel'] = _TWOFACTORCHANNEL
DESCRIPTOR.message_types_by_name['PreLoginResponse'] = _PRELOGINRESPONSE
DESCRIPTOR.message_types_by_name['LoginResponse'] = _LOGINRESPONSE
DESCRIPTOR.message_types_by_name['License'] = _LICENSE
DESCRIPTOR.message_types_by_name['OwnerlessRecord'] = _OWNERLESSRECORD
DESCRIPTOR.message_types_by_name['OwnerlessRecords'] = _OWNERLESSRECORDS
DESCRIPTOR.message_types_by_name['UserAuthRequest'] = _USERAUTHREQUEST
DESCRIPTOR.message_types_by_name['UidRequest'] = _UIDREQUEST
DESCRIPTOR.message_types_by_name['DeviceClientVersionUpdateRequest'] = _DEVICECLIENTVERSIONUPDATEREQUEST
DESCRIPTOR.message_types_by_name['ConvertUserToV3Request'] = _CONVERTUSERTOV3REQUEST
DESCRIPTOR.message_types_by_name['RevisionResponse'] = _REVISIONRESPONSE
DESCRIPTOR.enum_types_by_name['SupportedLanguage'] = _SUPPORTEDLANGUAGE
DESCRIPTOR.enum_types_by_name['LoginType'] = _LOGINTYPE
DESCRIPTOR.enum_types_by_name['DeviceStatus'] = _DEVICESTATUS
DESCRIPTOR.enum_types_by_name['LicenseStatus'] = _LICENSESTATUS
DESCRIPTOR.enum_types_by_name['AccountType'] = _ACCOUNTTYPE
DESCRIPTOR.enum_types_by_name['SessionTokenType'] = _SESSIONTOKENTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ApiRequest = _reflection.GeneratedProtocolMessageType('ApiRequest', (_message.Message,), dict(
  DESCRIPTOR = _APIREQUEST,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.ApiRequest)
  ))
_sym_db.RegisterMessage(ApiRequest)

ApiRequestPayload = _reflection.GeneratedProtocolMessageType('ApiRequestPayload', (_message.Message,), dict(
  DESCRIPTOR = _APIREQUESTPAYLOAD,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.ApiRequestPayload)
  ))
_sym_db.RegisterMessage(ApiRequestPayload)

Transform = _reflection.GeneratedProtocolMessageType('Transform', (_message.Message,), dict(
  DESCRIPTOR = _TRANSFORM,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.Transform)
  ))
_sym_db.RegisterMessage(Transform)

DeviceRequest = _reflection.GeneratedProtocolMessageType('DeviceRequest', (_message.Message,), dict(
  DESCRIPTOR = _DEVICEREQUEST,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.DeviceRequest)
  ))
_sym_db.RegisterMessage(DeviceRequest)

AuthRequest = _reflection.GeneratedProtocolMessageType('AuthRequest', (_message.Message,), dict(
  DESCRIPTOR = _AUTHREQUEST,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.AuthRequest)
  ))
_sym_db.RegisterMessage(AuthRequest)

NewUserMinimumParams = _reflection.GeneratedProtocolMessageType('NewUserMinimumParams', (_message.Message,), dict(
  DESCRIPTOR = _NEWUSERMINIMUMPARAMS,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.NewUserMinimumParams)
  ))
_sym_db.RegisterMessage(NewUserMinimumParams)

PreLoginRequest = _reflection.GeneratedProtocolMessageType('PreLoginRequest', (_message.Message,), dict(
  DESCRIPTOR = _PRELOGINREQUEST,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.PreLoginRequest)
  ))
_sym_db.RegisterMessage(PreLoginRequest)

LoginRequest = _reflection.GeneratedProtocolMessageType('LoginRequest', (_message.Message,), dict(
  DESCRIPTOR = _LOGINREQUEST,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.LoginRequest)
  ))
_sym_db.RegisterMessage(LoginRequest)

RegistrationRequest = _reflection.GeneratedProtocolMessageType('RegistrationRequest', (_message.Message,), dict(
  DESCRIPTOR = _REGISTRATIONREQUEST,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.RegistrationRequest)
  ))
_sym_db.RegisterMessage(RegistrationRequest)

DeviceResponse = _reflection.GeneratedProtocolMessageType('DeviceResponse', (_message.Message,), dict(
  DESCRIPTOR = _DEVICERESPONSE,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.DeviceResponse)
  ))
_sym_db.RegisterMessage(DeviceResponse)

Salt = _reflection.GeneratedProtocolMessageType('Salt', (_message.Message,), dict(
  DESCRIPTOR = _SALT,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.Salt)
  ))
_sym_db.RegisterMessage(Salt)

TwoFactorChannel = _reflection.GeneratedProtocolMessageType('TwoFactorChannel', (_message.Message,), dict(
  DESCRIPTOR = _TWOFACTORCHANNEL,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.TwoFactorChannel)
  ))
_sym_db.RegisterMessage(TwoFactorChannel)

PreLoginResponse = _reflection.GeneratedProtocolMessageType('PreLoginResponse', (_message.Message,), dict(
  DESCRIPTOR = _PRELOGINRESPONSE,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.PreLoginResponse)
  ))
_sym_db.RegisterMessage(PreLoginResponse)

LoginResponse = _reflection.GeneratedProtocolMessageType('LoginResponse', (_message.Message,), dict(
  DESCRIPTOR = _LOGINRESPONSE,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.LoginResponse)
  ))
_sym_db.RegisterMessage(LoginResponse)

License = _reflection.GeneratedProtocolMessageType('License', (_message.Message,), dict(
  DESCRIPTOR = _LICENSE,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.License)
  ))
_sym_db.RegisterMessage(License)

OwnerlessRecord = _reflection.GeneratedProtocolMessageType('OwnerlessRecord', (_message.Message,), dict(
  DESCRIPTOR = _OWNERLESSRECORD,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.OwnerlessRecord)
  ))
_sym_db.RegisterMessage(OwnerlessRecord)

OwnerlessRecords = _reflection.GeneratedProtocolMessageType('OwnerlessRecords', (_message.Message,), dict(
  DESCRIPTOR = _OWNERLESSRECORDS,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.OwnerlessRecords)
  ))
_sym_db.RegisterMessage(OwnerlessRecords)

UserAuthRequest = _reflection.GeneratedProtocolMessageType('UserAuthRequest', (_message.Message,), dict(
  DESCRIPTOR = _USERAUTHREQUEST,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.UserAuthRequest)
  ))
_sym_db.RegisterMessage(UserAuthRequest)

UidRequest = _reflection.GeneratedProtocolMessageType('UidRequest', (_message.Message,), dict(
  DESCRIPTOR = _UIDREQUEST,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.UidRequest)
  ))
_sym_db.RegisterMessage(UidRequest)

DeviceClientVersionUpdateRequest = _reflection.GeneratedProtocolMessageType('DeviceClientVersionUpdateRequest', (_message.Message,), dict(
  DESCRIPTOR = _DEVICECLIENTVERSIONUPDATEREQUEST,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.DeviceClientVersionUpdateRequest)
  ))
_sym_db.RegisterMessage(DeviceClientVersionUpdateRequest)

ConvertUserToV3Request = _reflection.GeneratedProtocolMessageType('ConvertUserToV3Request', (_message.Message,), dict(
  DESCRIPTOR = _CONVERTUSERTOV3REQUEST,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.ConvertUserToV3Request)
  ))
_sym_db.RegisterMessage(ConvertUserToV3Request)

RevisionResponse = _reflection.GeneratedProtocolMessageType('RevisionResponse', (_message.Message,), dict(
  DESCRIPTOR = _REVISIONRESPONSE,
  __module__ = 'APIRequest_pb2'
  # @@protoc_insertion_point(class_scope:Authentication.RevisionResponse)
  ))
_sym_db.RegisterMessage(RevisionResponse)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
